@@ -1,6 +1,6 @@
 ;;; site-lisp.el --- Manage site-lisp directories  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2021, 2022, 2023  Free Software Foundation, Inc.
+;; Copyright (C) 2021, 2022, 2023, 2024  Free Software Foundation, Inc.
 
 ;; Author: Philip Kaludercic <philipk@posteo.net>
 ;; Maintainer: Philip Kaludercic <~pkal/public-inbox@lists.sr.ht>
@@ -52,6 +52,10 @@
   "Name of file to store autoload forms in."
   :type 'string)
 
+(defcustom site-lisp-collect-recursivly nil
+  "Non-nil means that all files should be recursively scraped."
+  :type 'boolean)
+
 (defmacro site-lisp-generate-autoloads (dir file)
   "Generate autoloads for DIR as appropriate for the current version.
 The result should be stored in FILE."
@@ -65,6 +69,8 @@ The result should be stored in FILE."
 
 (defvar generated-autoload-file)
 
+(defvar site-lisp--autoload-file)
+
 ;;;###autoload
 (defun site-lisp-prepare (dir)
   "Byte-compile, prepare autoloads and note each directory in DIR.
@@ -73,15 +79,32 @@ of the list."
   (interactive "DPrepare: ")
   (if (listp dir)
       (mapc #'site-lisp-prepare dir)
-    (let ((backup-inhibited t)
-          (autoload-file (expand-file-name site-lisp-autoload-file dir)))
-      (dolist (dir (cons dir (directory-files dir t "^[^.]")))
+    (let* ((backup-inhibited t)
+           (site-lisp--autoload-file
+            (or (bound-and-true-p site-lisp--autoload-file)
+                (expand-file-name site-lisp-autoload-file dir)))
+           (old-modtime
+            ;; we check whatever the mtime of the autoload file was
+            ;; before we touched anything...
+            (file-attribute-modification-time
+             (file-attributes site-lisp--autoload-file))))
+      (dolist (dir (directory-files dir t "^[^.]"))
         (when (file-directory-p dir)
-          (message "Site-lisp: %s" dir)
           (add-to-list 'load-path dir)
-          (site-lisp-generate-autoloads dir autoload-file)))
+          (site-lisp-generate-autoloads
+           dir site-lisp--autoload-file)
+          (when (and site-lisp-collect-recursivly)
+            (site-lisp-prepare dir))))
+      (add-to-list 'load-path dir)
+      ;; ... and reset that after returning from a possible recursive
+      ;; descent through the file system, so that we don't ignore
+      ;; files that are now "accidentally" older than the just updated
+      ;; auto-load file.
+      (set-file-times site-lisp--autoload-file old-modtime)
+      (site-lisp-generate-autoloads
+       dir site-lisp--autoload-file)
       (byte-recompile-directory dir)
-      (load autoload-file nil t))))
+      (load site-lisp--autoload-file nil t))))
 
 (defun site-lisp-unprepare (dir)
   "Remove every directory in DIR from `load-path'.
